/*
 * randpass.c --- generates a random password, using MD5 as the random
 * 	number generator
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#include "hmac_md5.h"

#define MAX_SECRET_LENGTH  4096

static HMAC_MD5_CTX randpass_ctx;

static char randpass_tab[] =
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

int randpass_init(char *file_name)
{
	int		fd;
	int		count;
	unsigned char	buf[MAX_SECRET_LENGTH];

	fd = open(file_name, O_RDONLY);
	if (fd < 0)
		return errno;

	count = read(fd, buf, sizeof(buf));
	close(fd);
	if (count < 0)
		return errno;
	HMAC_MD5_SetKey(&randpass_ctx, buf, count);
	return 0;
}

int randpass_gen(char *password, int length)
{
	struct timeval	tv;
	int		i, b, ret;
	unsigned char	digest[16];

	ret = gettimeofday(&tv, 0);
	if (ret < 0)
		return errno;

	for (i = 0; i < length; i++) {
		b = i % sizeof(digest);
		if (b == 0)
			HMAC_MD5(&randpass_ctx, (unsigned char *) &tv,
				 sizeof(tv), digest);
		password[i] = randpass_tab[digest[b] % 64];
	}
	password[length] = 0;
	return 0;
}

#ifdef TEST
main(int argc, char **argv)
{
	int	i;
	char	password[9];
	
	randpass_init(TEST);
	for (i=0; i < 8; i++) {
		randpass_gen(password, 8);
		printf("Password %d: %s\n", i, password);
	}
	exit(0);
}
#endif
